"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-env browser */
exports.numCpus = navigator.hardwareConcurrency || 1;
function spawnWorker(workerCode) {
    const blob = new Blob([workerCode], { type: 'application/javascript' });
    const url = URL.createObjectURL(blob);
    return new Worker(url);
}
exports.spawnWorker = spawnWorker;
async function terminateWorker(worker) {
    await worker.worker.terminate();
}
exports.terminateWorker = terminateWorker;
function addWorkerListener(worker, type, listener) {
    // @ts-ignore
    worker.addEventListener(type, (event) => listener(event.data));
}
exports.addWorkerListener = addWorkerListener;
function makeWorkerCode(initString, workerString) {
    return `
const initFunction = ${initString};
const workerFunction = ${workerString};
const messagePort = self;

function postError(obj, error) {
  let errorToTransfer = error;
  if (error) {
    if (error.stack) {
      errorToTransfer = error.stack;
    } else if (error.message) {
      errorToTransfer = error.message;
    }
  }
  try {
    messagePort.postMessage(Object.assign({}, obj, {
      status: 'error',
      error: errorToTransfer
    }));
  } catch (e) {
    messagePort.postMessage({
      status: 'error',
      error: 'Work failed but error could not be transferred: ' + e.message
    });
  }
}

messagePort.onmessage = async (event) => {
  const { data: message } = event;
  if (message.type === 'work') {
    try {
      const result = await workerFunction(message.value);
      messagePort.postMessage({
        id: message.id,
        type: 'result',
        status: 'success',
        result
      });
    } catch (error) {
      postError({
        id: message.id,
        type: 'result'
      }, error);
    }
  } else if (message.type === 'init') {
    try {
      await initFunction(message.value);
      messagePort.postMessage({
        type: 'init',
        status: 'success'
      });
    } catch (error) {
      postError({
        type: 'init'
      }, error);
    }
  }
};
`;
}
exports.makeWorkerCode = makeWorkerCode;
